function [ak, y] = compLS_ak(s, f0, L, win, fs)
% Compute the parameters of HNM using Least Squares.
% 
%  ----INPUT PARAMETERS----
%    s: speech signal
%    f0: fundamental frequency of s
%    L: maximum harmonic
%    win: the window
%    fs: sampling frequency
%
%  ----OUTPUT PARAMENTERS----
%    ak: amplitude of harmonics
%    y: reconstructed signal
%
% (c) Yannis Pantazis, 2006, csd, uoc
%     mail: pantazis@csd.uoc.gr

N = (length(s)-1)/2; %length of s must be odd

k = -L:L;
n = -N:N;
ef0 = exp(2*pi*j*f0/fs);

E = ef0.^(n'*k); % dimension of E: (2N+1)x(2L+1)
W = diag(win);

ak = inv(E'*W*W*E)*E'*W*W*s; % solution of windowed LS

y = real(E*ak); % reconstructed signal

% plot(s.*win,'g');hold on;
% plot(y.*win,'k');hold off;
% plot((s-y).*win);
% pause;

