function [ak, bk, y] = compLS_akbk(s, f0, L, win, fs)
% Compute the parameters of time-varying HNM using Least Squares.
% 
%  ----INPUT PARAMETERS----
%    s: speech signal
%    f0: fundamental frequency of s
%    L: maximum harmonic
%    win: the window
%    fs: sampling frequency
%
%  ----OUTPUT PARAMENTERS----
%    ak: amplitude of harmonics
%    bk: slope of harmonics
%    y: reconstructed signal
%
% (c) Yannis Pantazis, 2006, csd, uoc
%     mail: pantazis@csd.uoc.gr


N = (length(s)-1)/2; %length of s must be odd

k = -L:L;
n = -N:N;
ef0 = exp(2*pi*j*f0/fs);

E = ef0.^(n'*k); % dimension of E: (2N+1)x(2L+1)
W = diag(win);
E = [E repmat(n',1,2*L+1).*E]; % dimension of E:(2N+1)x(4L+2)
%cos(pi*f0*n/fs)

x = inv(E'*W*W*E)*E'*W*W*s; % solution of LS

ak = x(1:2*L+1); % amplitudes
bk = x(2*L+2:4*L+2); % slopes

y = real(E*[ak;bk]); % reconstructed signal

% y1 = real(E(:,1:2*L+1)*ak); % reconstruction with amplitudes only
% y2 = real(E(:,2*L+2:4*L+2)*bk); % reconstruction with slope only

% plot(s,'g');hold on;
% plot(y,'k');hold off;
% plot(abs(s-y));
% pause;
