function compareBias
%
% This function is used to compare the bias obtained by the formula and the
% numerically estimated bias. Use of a 1rd order amplitude and phase time
% polynomial.

fs = 8000;
f = 2000;
N = 64;
T = N/fs;

dp = 30/T;

figure(1);
set(1,'Position',[1 600 650 600]);
%%%%%%%%%%%%   RECT WIN   %%%%%%%%%%%%%%
n = -N:N; % 2*T sec
win = rectwin(2*N+1);

A = 1;
p0 = pi/7;
p1 = -dp:dp/1000:dp;
L1 = length(p1);

fBias = p1-3*(1./(p1*T^2)-cot(p1*T)/T);

cBias = zeros(L1, 1);
df = zeros(L1, 1);
for k = 1:L1
    x = A*exp(j*p0) * exp(j*(2*pi*f*n/fs+p1(k)*n/fs));

    [ak, bk, ddf, y] = icompLS_akbk(x.', f, win, fs, 0);

    df(k) = 2*pi*ddf + fs*(imag(bk)*real(ak) - imag(ak)*real(bk))/abs(ak)^2;

    fBias(k) = p1(k) - df(k);
end
subplot(311);
plot(p1/(2*pi), fBias/(2*pi),'k', p1/(2*pi), cBias'/(2*pi),'k--');
xlabel('\eta_1  (Hz)\newline  (a)');ylabel('Bias(\eta_1)   (Hz)');legend('Eq. (30)', 'Numerical');
ylim([-4 4]*10^2);xlim([-400 400]);
idx = find(abs(p1)>abs(cBias') & abs(p1)<pi/T);
hold on;plot(p1(idx)/(2*pi), fBias(idx)/(2*pi),'k','lineWidth',2.5);hold off;

%%%%%%%%%%%%   HAMMING WIN   %%%%%%%%%%%%%%
win = hanning(2*N+1);

cBias = zeros(L1, 1);
df = zeros(L1, 1);
for k = 1:L1
    x = A*exp(j*p0) * exp(j*(2*pi*f*n/fs+p1(k)*n/fs));

    [ak, bk, ddf, y] = icompLS_akbk(x.', f, win, fs, 0);

    df(k) = 2*pi*ddf + fs*(imag(bk)*real(ak) - imag(ak)*real(bk))/abs(ak)^2;

    cBias(k) = p1(k) - df(k);
end
subplot(211);
plot(p1/(2*pi), fBias/(2*pi),'k', p1/(2*pi), cBias'/(2*pi),'k--');
xlabel('\eta_1 (Hz)\newline    (a)');ylabel('Bias(\eta_1)   (Hz)');legend('RectWin', 'Hamming');
ylim([-4 4]*10^2);xlim([-400 400]);
idx = find(abs(p1)>abs(cBias') & abs(p1)<3*pi/T);
hold on;plot(p1(idx)/(2*pi), cBias(idx)/(2*pi),'k--','lineWidth',2.5);hold off;


%%%%%%%%%%%%   ITERATIONS   %%%%%%%%%%%%%%
cBias1 = cBias;

cBias2 = zeros(L1, 1);
df = zeros(L1, 1);
for k = 1:L1
    x = A*exp(j*p0) * exp(j*(2*pi*f*n/fs+p1(k)*n/fs));

    [ak, bk, ddf, y] = icompLS_akbk(x.', f, win, fs, 2);

    df(k) = 2*pi*ddf + fs*(imag(bk)*real(ak) - imag(ak)*real(bk))/abs(ak)^2;

    cBias2(k) = p1(k) - df(k);
end
subplot(212);
plot(p1/(2*pi), cBias1/(2*pi),'k', p1/(2*pi), cBias2'/(2*pi),'k--');
xlabel('\eta_1  (Hz)\newline    (b)');ylabel('Bias(\eta_1)   (Hz)');legend('no iter', '2 iter');
ylim([-4 4]*10^2);xlim([-400 400]);

