function error_WN
%
% frequency + amplitude error for various frequency mismatch

K = 4;
fs = 8000;
N = 8*fs/1000;
n = -N:N;
win = hamming(2*N+1);

fk = [100, 200, 1000, 2000];
ak = [exp(j*pi/10), exp(j*pi/4), exp(j*pi/3), exp(j*pi/5)];

x = repmat(ak,2*N+1,1) .* exp(j*n'*2*pi*fk/fs);
x = sum(x, 2);


stdW = 10.^(-(-5:5:80)/20);
M = length(stdW);
NOI = 500;  % for 10000, results are the same; change it to 2000 for faster

freqEr = zeros(5, K, M);
amplEr = zeros(5, K, M);
for i = 1:NOI
    mis = 2*pi/fs*[20, 20, 200, 200].*(rand(1,4)-0.5); 

    for m = 1:M
        w = (randn(2*N+1,1) + j*randn(2*N+1,1))/sqrt(2);
        z = x + stdW(m) * w;

        wk = 2*pi*fk/fs;
        [a0, b0, Df0, y] = gcompLS_akbk(z, wk+mis, win, 0);
        [a1, b1, Df1, y] = gcompLS_akbk(z, wk+mis, win, 3);
        
        [aFFT, wFFT] = compFFT_parabolic(z, wk, ak);
        
        amplEr(1,:,m) = amplEr(1,:,m) + abs(a0.' - ak).^2/NOI;
        freqEr(1,:,m) = freqEr(1,:,m) + abs(mis-Df0).^2/NOI;
        amplEr(2,:,m) = amplEr(2,:,m) + abs(a1.' - ak).^2/NOI;
        freqEr(2,:,m) = freqEr(2,:,m) + abs(mis+Df1).^2/NOI; % why +?
        amplEr(5,:,m) = amplEr(5,:,m) + abs(aFFT - ak).^2/NOI;
        freqEr(5,:,m) = freqEr(5,:,m) + abs(wk-wFFT).^2/NOI;
    end
end

for i = 1:K
    SNRk = 10*log10((2*N+1)*abs(ak(i))^2./(stdW.^2));
    SNRk = SNRk-10*log10(2*N+1);
    figure(1);subplot(2,2,i);
    set(1,'Position',[1 600 750 550]);
    semilogy(SNRk, stdW.^2/(2*N+1),'k','lineWidth',2);hold on;
    semilogy(SNRk, reshape(amplEr(1,i,:),1,M),'k*-');
    semilogy(SNRk, reshape(amplEr(2,i,:),1,M),'ko-');
    semilogy(SNRk, reshape(amplEr(5,i,:),1,M),'k^-');
    hold off;
    h = gca;
    set(h,'FontSize',11);
    pos = get(h, 'outerPosition');
    if i==1 || i==3
        set(h,'OuterPosition',pos+[-0.06 -0.01 0.07 0.02]);
    else
        set(h,'OuterPosition',pos+[-0.01 -0.01 0.07 0.02]);
    end
    legend('CRB', 'no iter', '3 iter', 'FFT','Location','SouthWest');
    xlabel('SNR (dB)');ylabel(['MSE(a_' num2str(i) ')']);
    %xlim([20 100]);ylim([10^-10 1]);
    axis([-1 80 10^-10 1]);
    set(h,'XTick', [0 10 20 30 40 50 60 70 80]);
    %print -deps MSE_ampl

    figure(2);subplot(2,2,i);
    set(2,'Position',[815 600 750 550]);
    semilogy(SNRk, fs/2*pi*3*stdW.^2/(abs(ak(i))^2*N*(N+1)*(2*N+1)),'k','lineWidth',2);hold on;
    semilogy(SNRk, fs/2*pi*reshape(freqEr(1,i,:),1,M),'k*-');
    semilogy(SNRk, fs/2*pi*reshape(freqEr(2,i,:),1,M),'ko-');
    semilogy(SNRk, fs/2*pi*reshape(freqEr(5,i,:),1,M),'k^-');
    hold off;
    h = gca;
    set(h,'FontSize',11);
    pos = get(h, 'outerPosition');
    if i==1 || i==3
        set(h,'OuterPosition',pos+[-0.06 -0.01 0.07 0.02]);
    else
        set(h,'OuterPosition',pos+[-0.01 -0.01 0.07 0.02]);
    end
    legend('CRB', 'no iter', '3 iter', 'FFT','Location','SouthWest');
    xlabel('SNR (dB)');ylabel(['MSE(f_' num2str(i) ')']);
    %xlim([20 100]);ylim([10^-10 100]);
    axis([-1 80 10^-10 100]);
    set(h,'XTick', [0 10 20 30 40 50 60 70 80]);
    %print -deps MSE_freq
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ak, bk, Df, y] = gcompLS_akbk(s, wk, win, iter)
% Compute the parameters of time-varying HNM using Least Squares.
% 
%  ----INPUT PARAMETERS----
%    s: speech signal
%    wk: estimated frequencies
%    win: the window
%    iter: Number of iterations
%
%  ----OUTPUT PARAMENTERS----
%    ak: amplitude of harmonics
%    bk: slope of harmonics
%    y: reconstructed signal
%
% (c) Yannis Pantazis, 2006, csd, uoc
%     mail: pantazis@csd.uoc.gr


N = (length(s)-1)/2; %length of s must be odd
K = length(wk);
n = -N:N;

% INITIALIZATION
E = exp(j*n'*wk); % dimension of E: (2N+1)x(K)
W = diag(win);
E = [E repmat(n',1,K).*E]; % dimension of E:(2N+1)x(2K)

x = inv(E'*W*W*E)*E'*W*W*s; % solution of LS

ak = x(1:K); % amplitudes
bk = x(K+1:2*K); % slopes

Df = zeros(1,K);
for i = 1:iter
    Df = Df + ((imag(bk).*real(ak) - imag(ak).*real(bk))./abs(ak).^2)';

    E = exp(j*n'*(wk+Df)); % dimension of E: (2N+1)x(K)
    W = diag(win);
    E = [E repmat(n',1,K).*E]; % dimension of E:(2N+1)x(2K)

    x = inv(E'*W*W*E)*E'*W*W*s; % solution of LS

    ak = x(1:K); % amplitudes
    bk = x(K+1:2*K); % slopes
end

y = E*[ak;bk]; % reconstructed signal



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ak, wk] = compFFT_parabolic(z, wk, ak)
%
% Parameter estimation using peak picking and parabolic interpolation.

NFFT = 2048;
K = 16;
area = -K:K;
N = length(z);

win = hamming(length(z));
z = win.*z;

Z1 = fft([z((N+1)/2:N); zeros(NFFT-N, 1); z(1:(N-1)/2)], NFFT)/sum(win);
%Z1 = fft(z, NFFT)/sum(win);
Z = abs(Z1);

wk = round(wk*NFFT/(2*pi));
%plot(wk, abs(ak),'ro');hold on;

for i = 1:length(wk)
    [M, pos] = max(Z(wk(i)+area));
    
    C = Z(wk(i)+pos-K-1);
    B = (Z(wk(i)+pos-K) - Z(wk(i)+pos-K-2))/2;
    A = (Z(wk(i)+pos-K) -2*Z(wk(i)+pos-K-1) + Z(wk(i)+pos-K-2))/2;
    
    w_min = - B/(2*A);
    ak(i) = (A*w_min^2 + B*w_min + C)*exp(j*angle(Z1(wk(i)+pos-K-2)));
    wk(i) = wk(i)+pos-K-2 + w_min;
%    plot(0:NFFT/2-1,Z(1:NFFT/2));hold on;plot(wk(i), abs(ak(i)),'g*');pause;
end
%hold off;

wk = wk*2*pi/NFFT;

