function [amplEr, freqEr] = error_spacing
%
% frequency + amplitude error for various frequency mismatch

K = 4;
fs = 8000;
N = 8*fs/1000;
n = -N:N;
win = hamming(2*N+1);

fk = [1000, 1000, 1000, 2000];
ak = [exp(j*pi/10), exp(j*pi/4), exp(j*pi/3), exp(j*pi/5)];

df = (60:20:200);
M = length(df);
NOI = 1000;
freqEr = zeros(K, M, 5);
amplEr = zeros(K, M, 5);    

stdW = 10.^(-50/20);

for m=1:M
    N = round(2*fs/df(m));
    n = -N:N;
    win = hamming(2*N+1);
    
    fk = [1000-df(m), 1000, 1000+df(m), 2000];
    fprintf(1, 'fk = [%f %f %f %f]', fk(1), fk(2), fk(3), fk(4));
    x = repmat(ak,2*N+1,1) .* exp(j*n'*2*pi*fk/fs);
    x = sum(x, 2);    
    c = 0;
    for i = 1:NOI
        w = (randn(2*N+1,1) + j*randn(2*N+1,1))/sqrt(2);
        z = x + stdW * w;
        nok = 1;
        while nok
            mis = 4*pi/fs*[0.1*df(m), 0.1*df(m), 0.1*df(m), 40].*(rand(1,4)-0.5);
            wk = 2*pi*fk/fs;
            [a0, b0, Df0, y] = gcompLS_akbk(z, wk+mis, win, 0);                
            [a1, b1, Df1, y] = gcompLS_akbk(z, wk+mis, win, 3);
            if (norm(a0)>0) & (norm(a1)>0)
                nok = 0;
            else
                c = c+1;
            end
            %        [a2, b2, Df2, y] = gcompLS_akbk(z, wk+mis, win, 4);
            %        [a5, b5, Df5, y] = gcompLS_akbk(z, wk+mis, win, 10);
        end
        
        [aFFT, wFFT] = compFFT_parabolic(z, wk, ak);
        % plot(abs(z).*win);hold on;plot(abs(y).*win,'r');plot(abs(z-y).*win,'g');pause;
        amplEr(:,m,1) = amplEr(:,m,1) + abs(a0 - ak.').^2;
        freqEr(:,m,1) = freqEr(:,m,1) + abs(mis'+Df0').^2;
        amplEr(:,m,2) = amplEr(:,m,2) + abs(a1 - ak.').^2;
        freqEr(:,m,2) = freqEr(:,m,2) + abs(mis'+Df1').^2;
%        amplEr(3,:,m) = amplEr(3,:,m) + abs(a2.' - ak).^2/NOI;
%        freqEr(3,:,m) = freqEr(3,:,m) + abs(mis+Df2).^2/NOI;
%        amplEr(4,:,m) = amplEr(4,:,m) + abs(a5.' - ak).^2/NOI;
%        freqEr(4,:,m) = freqEr(4,:,m) + abs(mis+Df5).^2/NOI;
        amplEr(:,m,5) = amplEr(:,m,5) + abs(aFFT.' - ak.').^2;
        freqEr(:,m,5) = freqEr(:,m,5) + abs(wk.'-wFFT.').^2;
        
%        plot(abs(ak));hold on;plot(abs(a0),'r');plot(abs(a5),'g');hold off;
    end
    amplEr(:,m,1) = sqrt(amplEr(:,m,1) / NOI);
    freqEr(:,m,1) = sqrt(freqEr(:,m,1) / NOI)*fs/(2*pi);
    amplEr(:,m,2) = sqrt(amplEr(:,m,2) / NOI);
    freqEr(:,m,2) = sqrt(freqEr(:,m,2) / NOI)*fs/(2*pi);
    amplEr(:,m,5) = sqrt(amplEr(:,m,5) / NOI);
    freqEr(:,m,5) = sqrt(freqEr(:,m,5) / NOI)*fs/(2*pi);
    fprintf(1, ' : %d cas mal conditionnes / %d\n', c, NOI);
end

figure(1);
for k=1:4
    subplot(2,2,k);
    semilogy(df, freqEr(k,:,1), 'k*-');
    hold on
    semilogy(df, freqEr(k,:,2), 'ko-');
    semilogy(df, freqEr(k,:,5), 'k^-');
    hold off
end



        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ak, y] = gcompLS_ak(s, wk, win)
% Compute the parameters of HNM using Least Squares.
% 
%  ----INPUT PARAMETERS----
%    s: speech signal
%    wk: estimated frequencies
%    win: the window
%
%  ----OUTPUT PARAMENTERS----
%    ak: amplitude of sinusoids
%    y: reconstructed signal
%
% (c) Yannis Pantazis, 2008, csd, uoc
%     mail: pantazis@csd.uoc.gr

N = (length(s)-1)/2; %length of s must be odd
n = -N:N;

E = exp(j*n'*wk); % dimension of E: (2N+1)x(K)
W = diag(win);

ak = inv(E'*W*W*E)*E'*W*W*s; % solution of windowed LS

y = E*ak; % reconstructed signal

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ak, bk, Df, y] = gcompLS_akbk(s, wk, win, iter)
% Compute the parameters of time-varying HNM using Least Squares.
% 
%  ----INPUT PARAMETERS----
%    s: speech signal
%    wk: estimated frequencies
%    win: the window
%    iter: Number of iterations
%
%  ----OUTPUT PARAMENTERS----
%    ak: amplitude of harmonics
%    bk: slope of harmonics
%    y: reconstructed signal
%
% (c) Yannis Pantazis, 2006, csd, uoc
%     mail: pantazis@csd.uoc.gr


N = (length(s)-1)/2; %length of s must be odd
K = length(wk);
n = -N:N;

% INITIALIZATION
E = exp(j*n'*wk); % dimension of E: (2N+1)x(K)
W = diag(win);
E = [E repmat(n',1,K).*E]; % dimension of E:(2N+1)x(2K)
E1 = E'*W*W*E;
if rcond(E1)<1e-8
    Df = 0;
    ak = 0;
    bk = 0;
    y = 0;
    return;
else
    x = inv(E1)*E'*W*W*s; % solution of LS
end

ak = x(1:K); % amplitudes
bk = x(K+1:2*K); % slopes

Df = zeros(1,K);
for i = 1:iter
    Df = Df + ((imag(bk).*real(ak) - imag(ak).*real(bk))./abs(ak).^2)';

    E = exp(j*n'*(wk+Df)); % dimension of E: (2N+1)x(K)
    W = diag(win);
    E = [E repmat(n',1,K).*E]; % dimension of E:(2N+1)x(2K)
    E1 = E'*W*W*E;
    if rcond(E1)<1e-8
        Df = 0;
        ak = 0;
        bk = 0;
        y = 0;
        return;
    else
        x = inv(E1)*E'*W*W*s; % solution of LS
    end
    ak = x(1:K); % amplitudes
    bk = x(K+1:2*K); % slopes
end

y = E*[ak;bk]; % reconstructed signal


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ak, wk] = compFFT_parabolic(z, wk, ak)
%
% Parameter estimation using peak picking and parabolic interpolation.

NFFT = 2048;
K = 12; %YP: K=16
area = -K:K;
N = length(z);

win = hamming(length(z));
z = win.*z;

Z1 = fft([z((N+1)/2:N); zeros(NFFT-N, 1); z(1:(N-1)/2)], NFFT)/sum(win);
%Z1 = fft(z, NFFT)/sum(win);
Z = abs(Z1);

wk = round(wk*NFFT/(2*pi));
%plot(wk, abs(ak),'ro');hold on;

for i = 1:length(wk)
    [M, pos] = max(Z(wk(i)+area));
    
    C = Z(wk(i)+pos-K-1);
    B = (Z(wk(i)+pos-K) - Z(wk(i)+pos-K-2))/2;
    A = (Z(wk(i)+pos-K) -2*Z(wk(i)+pos-K-1) + Z(wk(i)+pos-K-2))/2;
    
    w_min = - B/(2*A);
    ak(i) = (A*w_min^2 + B*w_min + C)*exp(j*angle(Z1(wk(i)+pos-K-2)));
    wk(i) = wk(i)+pos-K-2 + w_min;
    %plot(0:NFFT/2-1,Z(1:NFFT/2));hold on;plot(wk(i),
    %abs(ak(i)),'g*');pause;
end
%hold off;

wk = wk*2*pi/NFFT;

